## Copyright (C) 2007 Sam Robson
##
## This file is part of Envisage.
##
## Envisage is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Envisage is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Envisage; if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 


###############################################################################
###############################################################################
##                      BEGIN - FUNCTION - envisage                          ##
###############################################################################
###############################################################################

envisage <- function(expData, widget=TRUE, MTC=NULL, pCutoff=NULL, 
    useParams=NULL, param4INT=NULL, paramType=NULL, fileResults=getwd(), 
    MEorINT="INT", startModel=NULL)
{

###############################################################################
#                          1 - GETTING THE DATA IN                            #
###############################################################################		

## Check widget argument specified correctly
if (!widget %in% c(TRUE, FALSE)) {
    stop("Please ensure that argument 'widget' is TRUE or FALSE")
}
		
## Extract gene names and number of genes in the experiment
gNames <- featureNames(expData)
nGenes <- length(gNames)
		
## Extract sample names and number of samples in the experiment
sNames <- sampleNames(expData)
nSamples <- length(sNames)
		
## Extract the variable information		
paramData <- pData(expData)
startParamNames <- colnames(paramData)
paramNames <- startParamNames	
startNParam <- length(startParamNames)
nParam <- startNParam

## Ensure that any variable levels labelled "NA" by the user are
## not considered to be missing data
paramMatrix <- as.matrix(paramData)
for (i in 1:nSamples) {
    for (j in 1:nParam) {
        if(is.na(paramMatrix[i,j])) paramMatrix[i,j] <- as.character("NAN")
    }
}
rm(i,j)
paramData <- as.data.frame(paramMatrix)

## Extract gene expression values from data object
d <- dataTransform(expData)


###############################################################################
#                       2 - REMOVING UNWANTED VARIABLES                       #
###############################################################################

## Flag any labelling variables
labParams <- paramNames[labelParams(paramData)]

## Remove variables with a constant value
constParam <- NULL
for (i in 1:nParam) {
    if (length(levels(paramData[[i]])) == 1) {
        constParam <- c(constParam, i)
    }
}
rm(i)
if (length(constParam) != 0) {
    paramData <- paramData[, -constParam]
}
nParam <- length(paramData)
paramNames <- names(paramData)
        
## Avoid problems with R objects whose name begins with a number by prefixing
## the name with the character "X"
for (i in 1:nParam) {
    if (substr(names(paramData)[i], 1, 1) %in% 0:9) {
        names(paramData)[i] <- paste("X", names(paramData)[i], sep="")
    }
}
rm(i)

## Ensure variable names do not contain "-" or "\" or "/" by removing them
names(paramData) <- gsub("-", "_", names(paramData))
names(paramData) <- gsub("/", "_", names(paramData))
names(paramData) <- gsub(" ", "_", names(paramData))

## Extract adjusted variable names
paramNames <- names(paramData)

## Flag any highly correlated variables
cor <- corParams(paramData)
corParamNames <- cor[[1]]
corParams <- cor[[2]]

## If widget selected
if (widget==TRUE) {

###############################################################################
#                                3 - LAUNCH GUI                               #
###############################################################################

## Load up additional tcl/tk packages
tclRequire("BWidget")
tclRequire("Iwidgets")

## Specify fonts
fontLabelsL <- tkfont.create(family="times", size=14, weight="bold")
fontLabels <- tkfont.create(family="times", size=12, weight="bold")
fontLabelsSmall <- tkfont.create(family="times", size=10, weight="bold")
fontTiny <- tkfont.create(family="times", size=3, weight="bold")
fontNames <- tkfont.create(family="times", size=12)
fontHeader <- tkfont.create(family="times", size=26, weight="bold")

## Set up top level frame environment
tt <- tktoplevel()
tktitle(tt) <- "Envisage GUI"

## Main GUI frame
frameOverall <- tkframe(tt)

## Frame for GUI title header
frameHeader <- tkframe(frameOverall, relief="flat", borderwidth=2)
    tkgrid(tklabel(frameHeader, text="Envisage", font=fontHeader))
    tkgrid(tklabel(frameHeader, text=paste("Enable Numerous Variables ",
        "In Significance Analysis of Gene Expression", sep=""), 
        font=fontLabels))
tkgrid(frameHeader, columnspan=2)

## Frame for selecting variables and specifying their class (numeric 
## or categoric)
frameParam <- tkwidget(frameOverall, "iwidgets::labeledframe", 
    labelpos="nw", labeltext="Select Variable Classes for Modelling")

    ## Column headings
    titleFrame <- tkframe(frameParam, borderwidth=2, width=400)
        labNone <- tklabel(titleFrame, text="Exclude", 
            font=fontLabelsSmall, width=8)
        labCat <- tklabel(titleFrame, text="Numeric", 
            font=fontLabelsSmall, width=7)
        labNum <- tklabel(titleFrame, text="Categoric", 
            font=fontLabelsSmall, width=8)
        blankLab <- tklabel(titleFrame, text="  ", width=30)
        tkgrid(blankLab, labNone, labCat, labNum)
    tkgrid(titleFrame)

    ## Scrollable frame for variable selection
    sw <- tkwidget(frameParam, "ScrolledWindow", relief="sunken", 
        borderwidth=2,bg="white")
    sf <- tkwidget(sw, "ScrollableFrame", width=400, height=200)
    tcl(sw, "setwidget",sf)
    subfID <- tclvalue(tcl(sf, "getframe"))

    usePValue <- list()
    paramLabel <- list()
    rbValue <- list()
    radSel <- list()
    radCat <- list()
    radNum <- list()
    frameBox <- list()
    pLab <- list()

    ## Assign check boxes and radioboxes for each variable
    for (i in 1:nParam) 
    {
        ## Frame for selection of individual variables
        frameBox[[i]] <- tcl("frame", paste(subfID, ".", i, sep=""), 
            relief="flat", borderwidth=2)

        ## Give each variable a label
        pLab[[i]] <- tcl("label", paste(subfID, ".", i, ".1", sep=""), 
            text=paste("p", i, sep=""), font=fontLabels, width=3)

        ## Variable name
        paramLabel[[i]] <- tcl("label", paste(subfID, ".", i, ".3", 
            sep=""), text=corParamNames[i], font=fontLabels, width=20)

        ## Class radio buttons
        rbValue[[i]] <- tclVar("None")

        ## To select variables
        radSel[[i]] <- tcl("radiobutton", paste(subfID,
            ".", i, ".4", sep=""), width=5)
        tkconfigure(radSel[[i]], variable=rbValue[[i]],
            value="None")
       
        ## For numeric class
        radNum[[i]] <- tcl("radiobutton", paste(subfID,
            ".", i, ".5", sep=""), width=5)
        tkconfigure(radNum[[i]], variable=rbValue[[i]],
            value="numeric")

        ## For categoric class
        radCat[[i]] <- tcl("radiobutton", paste(subfID,
            ".", i, ".6", sep=""), width=5)
        tkconfigure(radCat[[i]], variable=rbValue[[i]],
            value="categoric")

        ## Combine
        tkgrid(pLab[[i]], paramLabel[[i]], 
            radSel[[i]], radNum[[i]], radCat[[i]])
        tkconfigure(paramLabel[[i]], justify="left")
        tkgrid(frameBox[[i]])
    }  
    tkgrid(sw)
    tkgrid(tklabel(frameParam, text=paste("Please Note: *, **, etc. ",
        "indicates that model variables are highly correlated. Please ensure\n",
        "that only one of each set is chosen before continuing.",sep=""),
        justify="left"), sticky="w")

    ## Function to reset selection of variable classes
    resetAll <- function() {
        for (i in 1:nParam) {    
            rbValue[[i]] <<- tclVar("None")
            tkconfigure(radSel[[i]], variable=rbValue[[i]])
            tkconfigure(radNum[[i]], variable=rbValue[[i]])
            tkconfigure(radCat[[i]], variable=rbValue[[i]])
        }
    }

    ## Allow user to check or clear all boxes in one go
    frameSelect <- tkframe(frameParam, relief="groove", borderwidth=2)
        resetBut <- tkbutton(frameSelect, text="Reset",
            command=resetAll, bg="grey")
        tkgrid(resetBut)
    tkgrid(frameSelect)

## Main frame for selecting arguments for modelling
frameArg <- tkwidget(frameOverall, "iwidgets::labeledframe", 
    labelpos="nw", labeltext="Model Arguments")

    ## Frame for selecting multiple testing correction
    frameMTC <- tkframe(frameArg, relief="groove", borderwidth=2)
        tkgrid(tklabel(frameMTC, text="Multiple Testing Correction", 
            font=fontLabelsSmall), sticky="w")
        mtcorrect <- c("None", "Benjamini & Hochberg (FDR)", "Bonferroni (FWER)", 
            "Benjamini & Yekutieli (FWER)", "Holm (FWER)", "Hochberg (FWER)",
            "SidakSS (FWER)", "SidakSD (FWER)")
        mtcSelect<-tklistbox(frameMTC, height=7, width=25,
            selectmode="single", background="white")
        tkgrid(mtcSelect)
        for (i in (1:7))
        {
            tkinsert(mtcSelect, "end", mtcorrect[i])
        }
        tkselection.set(mtcSelect, 1)
    tkgrid(frameMTC,sticky="nsew")

    ## Select whether to model using main effect terms or first order
    ## interactions
    frameMEorINT <- tkframe(frameArg, relief="groove", borderwidth=2)
        tkgrid(tklabel(frameMEorINT, text="Model Terms", font=fontLabelsSmall), 
            columnspan=2, sticky="w")
        radME <- tkradiobutton(frameMEorINT)
        radINT <- tkradiobutton(frameMEorINT)
        rbValINT <- tclVar("INT")
        tkconfigure(radME, variable=rbValINT, value="ME") 
        tkconfigure(radINT, variable=rbValINT, value="INT") 
        tkgrid(tklabel(frameMEorINT, text="Main Effect Terms "), radME)
        tkgrid(tklabel(frameMEorINT, text="1st Order Interactions "), radINT)
    tkgrid(frameMEorINT,sticky="nsew")

    ## Allow user to specify a model to specify which variables to force
    ## into the model
    frameM <- tkframe(frameArg, relief="groove", borderwidth=2)
        tkgrid(tklabel(frameM, text="Starting Model", font=fontLabelsSmall), 
            columnspan=2, sticky="w")
        Minput <- tclVar("")
        Mentry <- tkentry(frameM, textvariable=Minput)
        tkgrid(tklabel(frameM, text="x ~ ", font=fontLabels), Mentry)
    tkgrid(frameM, sticky="nsew")

    ## Specify the p value cutoff level for significance
    framePCutoff <- tkframe(frameArg, relief="groove", borderwidth=2)
        labelP <- tklabel(framePCutoff, text="P Value Cutoff", font=fontLabelsSmall)
        p <- tclVar("0.05")
        pEntry <- tkentry(framePCutoff, textvariable=p, width=25)
        tkgrid(labelP, sticky="w")
        tkgrid(pEntry, sticky="w")
    tkgrid(framePCutoff, sticky="nsew")
    
## Combine variable and argument selection frames
tkgrid(frameParam,frameArg,sticky="nsew")

## Specify the file in which to save the results
frameFileSelect <- tkwidget(frameOverall, "iwidgets::labeledframe", 
    labelpos="nw", labeltext="Select folder in which to save results:")
   
    ## Browse for file to save results and update the entry box in 
    ## the GUI
    browse <- function() {
        file <- tclVar(tkchooseDirectory())
        tkconfigure(fileInput, textvariable=file)
    }
    file <- tclVar(as.character(getwd()))
    browseLabel <- tklabel(frameFileSelect, text="Folder Name:")
    browseBut <- tkbutton(frameFileSelect, text="Browse", command=browse)
    fileInput <- tkentry(frameFileSelect, width=80, textvariable=file)
    tkgrid(browseLabel, fileInput, browseBut, columnspan=2)
tkgrid(frameFileSelect, sticky="nsew", columnspan=3)

## Progress report for analysis
frameProgress <- tkframe(frameOverall, relief="groove", borderwidth=2, width=400)
    progress <- tclVar("0")
    progressLabel <- tklabel(frameProgress, textvariable = progress)
    progressLabel2 <- tkwidget(frameProgress, "ProgressBar", 
        variable=progress, width=450, height=20, maximum=nGenes)
    tkgrid(tklabel(frameProgress, font=fontLabels, text="Progress:\t"), 
        progressLabel, tklabel(frameProgress, text=paste(" of ", nGenes, 
        " Genes\t", sep="")), progressLabel2)
tkgrid(frameProgress, sticky="nsew", columnspan=3)


###############################################################################
###############################################################################
##                                                                           ##
##                             Function - OnOK                               ##
##                                                                           ##
###############################################################################
###############################################################################

## Function that runs when the OK button on the main GUI window is pressed
OnOK <- function()
{ 
    ## Switch on hourglass cursor
    hgOn()
    
    ###########################################################################
    #                      4 - PULL OUT ARGUMENT VALUES                       #
    ###########################################################################
   
    ## Get selected multiple testing correction from GUI. If no MTC is
    ## selected, show a warning and go back to the GUI window
    mtcShort <- c("None", "BH", "BY", "Bonferroni", "Holm", "Hochberg",
        "SidakSS", "SidakSD")
    MTC <- mtcShort[as.numeric(tkcurselection(mtcSelect))+1]
    if (length(MTC) == 0) {
        tkmessageBox(title="No MTC Selected", 
            message=paste("Please ensure that you have ",
            "selected a multiple testing\ncorrection from the list, ",
            "or select 'None' if no multiple testing\ncorrection is ",
            "required.", sep=""), type="ok", icon="warning")

        ## Hourglass cursor off
        hgOff()
    }

    ## Get selected P cutoff from GUI
    pCutoff <- as.numeric(tclvalue(p))
   
    ## Ensure that p value is between 0 and 1
    if (pCutoff < 0 || pCutoff > 1) {
        warning <- tkmessageBox(title="P Value Cutoff Out of Range",
            message="p value cutoff must be between 0 and 1.", icon="warning",
            type="ok")
        
        ## Hourglass cursor off
        hgOff()
    }

    ## Main effects or interactions
    MEorINT <- tclvalue(rbValINT)
   
    ## Specify which variables to use in the modelling, and specify the 
    ## class of those variables
    useParams <- NULL
    paramType <- NULL
    nNone <- 0
    for (i in 1:nParam) 
    {
        if (tclvalue(rbValue[[i]]) != "None") {
            useParams <- c(useParams, paramNames[i])
        } else nNone <- nNone + 1
        paramType <- c(paramType, tclvalue(rbValue[[i]]))
    }
    
    ## If no variables selected as numeric or categoric, stop
    if (nNone == nParam) {
        tkmessageBox(title="No variables selected", message=paste("You have",
            "chosen to exclude all variables. Please select variables for",
            "analysis as 'Categoric' or 'Numeric'."), icon="warning", 
            type="ok")

        ## Hourglass cursor off
        hgOff()
    }

    ## File to save results to
    fileResults <- tclvalue(tkget(fileInput))
    
    ## Ensure that highly correlated variables are not selected
    if (length(corParams) != 0) {
        for (i in 1:length(corParams)) {
            count <- 0
            for (j in 1:length(corParams[[i]])) {
                if (startParamNames[corParams[[i]][j]] %in% useParams) {
                    count <- count +1
                }
            }

            ## Warning message if highly correlated variables are
            ## selected for modelling
            if (count > 1) {
                message <- c("Warning: The variables:\n\n")
                for (j in 1:length(corParams[[i]])) {
                    if (startParamNames[corParams[[i]][j]] %in% useParams) {
                        message <- paste(message, "\t", 
                            startParamNames[corParams[[i]][j]], "\n", sep="")
                    }
                }
                rm(j)
                message <- paste(message, "\nshow very high ",
                    "correlation. Please ensure that only\none of ",
                    "these is selected before continuing.", sep="")           
                warning <- tkmessageBox(title=paste("Highly Correlated ",
                    "Variables Selected", sep=""), message=message, 
                    icon="warning", type="ok")
           
                ## Hourglass cursor off
                hgOff()
            }
        }
    }
    
    ## Window showing the selected variable classes
    selVar <- NULL
    for (i in 1:length(paramType)) {
        if (paramType[i] == "categoric") {
            selVar <- paste(selVar, paramNames[i], " - categoric\n", sep="")
        } else if (paramType[i] == "numeric") {
            selVar <- paste(selVar, paramNames[i], " - numeric\n", sep="")
        }
    }
    vars <- tkmessageBox(title="Selected variable classes", 
        message=paste("Would you like to continue with the following selected",
        " variables?\n\n", selVar, sep=""), type="yesno", icon="question")   

    if (tclvalue(vars) == "yes") {
    
    ## Switch on hourglass cursor
    hgOn()

    ###########################################################################
    #                     5 - ASSIGN CLASSES TO PARAMETERS                    #
    ###########################################################################

    ## Ensure that numerical variables are not stored as factors in the design 
    ## matrix
    adjParamData <- paramData		
    for (i in 1:nParam) {
        if (paramType[i] == "numeric") {
	    adjParamData[[i]] <- as.numeric(as.character(paramData[[i]]))
        }
    }


    ###########################################################################
    #         6 - ENSURE THAT LABELLING PARAMETERS ARE REMOVED                #
    ###########################################################################

    ## Ensure that no labelling variables are selected
    if (length(labParams) != 0) {
        warnLabParams <- NULL
        for (i in 1:length(labParams)) {
            if (labParams[i] %in% useParams && 
                !is.numeric(adjParamData[[labParams[i]]])) {
                warnLabParams <- c(warnLabParams, labParams[i])
            }
        }
        rm(i)

        ## Warn user about labelling variables
        if (length(warnLabParams) != 0) {
            message <- c("Warning: The variables:\n\n")

            ## List possible labelling variables
            for (i in 1:length(warnLabParams)) {
                message <- paste(message, "\t", warnLabParams[i],
                    "\n", sep="")
            }
            rm(i)

            ## Warning message
            message <- paste(message,"\nhave unique values for each sample.\n\n",
                "If the above variables are numeric, please ensure that you ",
                "have\nselected the 'numeric' radio button.\n\nIf they are ",
                "labelling variables, then remove them completely from\nthe ",
                "modelling procedure to avoid overfitting.", sep="")           
            warning <- tkmessageBox(title="Labelling Variables Selected",
                message=message, icon="warning", type="ok")
           
            ## Hourglass cursor off
            hgOff()
        }
    }

    ## Extract updated variable information
    paramNames <- useParams
    nParam <- length(paramNames)

    ## If only one variable chosen, treat as only main effect
    if (nParam == 1) MEorINT <- "ME"

    ## Remove unwanted variables from adjParamData
    adjParamData <- as.data.frame(adjParamData[, paramNames])
    names(adjParamData) <- paramNames

    ## Create starting model for the analysis
    tempModel <- modelEntry(pData=paramData, Mentry=Mentry, 
        useParams=useParams, labParams=labParams, MEorINT=MEorINT)
    startingModel <- as.formula(tempModel[[1]])
    intPos <- tempModel[[2]]

    if (length(intPos) != 0 && MEorINT == "ME") {
        tkmessageBox(title="No interactions to calculate", 
        message=paste("Warning: Your starting model contains ",
        "interaction terms,\nbut you have selected to only model ",
        "main effect terms.\nPlease either remove the interaction ",
        "terms from the starting\nmodel, or select '1st Order ",
        "Interactions'.", sep=""), type="ok", icon="warning")

        ## Hourglass cursor off
        ## Function that changes the cursor back to the default
        hgOff()  
    }     

 
    ###########################################################################
    #                      7 - CHECK INTERACTION LEVELS                       #
    ###########################################################################
    
    ## This section calculates the number of levels in the model terms (first 
    ## order interaction terms and main effects) and compares this to the number 
    ## of samples. If there are more terms than samples, we may see overfitting 
    ## of the data, so the user is given the option to select variables for 
    ## which only main effect terms are required. It is recommended to do this 
    ## with any variables with a large number of levels to avoid problems 
    ## further down the line.
    
    param4INT <- NULL

    ###########################################################################
    #                      A - INT ONLY MODEL CALCULATION                     #
    ###########################################################################	

    ## Calculate total number of factor levels for first order interactions
    if (MEorINT == "INT") {
        allLevels <- INTlevels(adjParamData)

        ## Allows you to select which variables to use for interactions
        tt2 <- tktoplevel()
        tktitle(tt2) <- "First order interaction selection"
        mainFrame2 <- tkframe(tt2)
        fontLab2 <- tkfont.create(family="times",size=12,weight="bold")

        frameTitle2 <- tkframe(mainFrame2)
        tkgrid(tklabel(frameTitle2, text="Variable\nName", width=15, 
            font=fontLab2), tklabel(frameTitle2, text="Degrees of\nFreedom", 
            width=10, font=fontLab2), tklabel(frameTitle2, 
            text="Select\nVariable", width=10, font=fontLab2))
        tkgrid(frameTitle2,sticky="w")

        ## Scrollable frame for variable selection
        sw2 <- tkwidget(mainFrame2,"ScrolledWindow",
            relief="sunken",borderwidth=2)
        sf3 <- tkwidget(sw2, "ScrollableFrame", width=320)
        tcl(sw2,"setwidget",sf3)
        subfID3 <- tclvalue(tcl(sf3,"getframe"))
    
        frameBox2 <- list()
        paramName2 <- list()
        outputBox2 <- list()
        outBoxValue2 <- list()
        levelInfo2 <- list()

        ## Create scrolled frame with all variables and interactions
        for (i in 1:length(adjParamData)) 
        {
            frameBox2[[i]] <- tcl("frame", paste(subfID3,".",i,sep=""), 
            relief="flat", borderwidth=2)

            ## Variable name
            paramName2[[i]] <- tcl("label", paste(subfID3,".",i,".1",sep=""), 
                text=names(adjParamData)[i], width=20, justify="left")

            ## Levels for each variable
            levelInfo2[[i]] <- tcl("label", paste(subfID3,".",i,".2",sep=""), 
                text=allLevels[i], width=15)

            ## Select output variables
            outputBox2[[i]] <- tcl("checkbutton", paste(subfID3,
                ".",i,".4",sep=""), width=10)
            tkconfigure(outputBox2[[i]], cursor="hand2")
            outBoxValue2[[i]] <- tclVar("1")
            tkconfigure(outputBox2[[i]], variable=outBoxValue2[[i]])

            ## Combine
            tkgrid(paramName2[[i]], levelInfo2[[i]], outputBox2[[i]])
            tkgrid(frameBox2[[i]])
        }
        rm(i)
        tkgrid(sw2)

###############################################################################
##                                       OnOK2                               ##
###############################################################################
        OnOK2 <- function() {
            hgOn()
            options(show.error.messages=TRUE)
            param4INT <- NULL
            for (i in 1:length(adjParamData)) {
                if (tclvalue(outBoxValue2[[i]]) == "1") {
                    param4INT <- c(param4INT,names(adjParamData)[i])
                }
            }
            rm(i)
     
            ## If no variables selected for interactions, treat as main 
            ## effects
            if (length(param4INT) == 0) {
                MEorINT <- "ME"
            }

            ## All selected variable and interaction terms  
            allINT <- gsub(" ", "_", param4INT)
            if (length(param4INT) != 0) { 
            for (i in 1:length(param4INT)) {
                for (j in 1:length(param4INT)) {
                    if (j > i && param4INT[j] != param4INT[i]) {
	                allINT <- c(allINT, gsub(" ", "_", 
	                    paste(param4INT[i], ":", param4INT[j], sep="")))
                    }
	              }
            }
            rm(i,j)

            ## All levels for chosen variable interactions
            allINTLevels <- NULL
            temp <- NULL
            for (i in 1:length(allINT)) {
                allINTLevels <- c(allINTLevels, as.numeric(allLevels[allINT[i]]))
                temp <- c(temp, names(allLevels[allINT[i]]))
            }
            rm(i)
            names(allINTLevels) <- temp
            } else allINTLevels <- allLevels[1:nParam]

            ## If still too many levels, try again
            if (sum(allINTLevels) > nrow(adjParamData)) {
                options(show.error.messages=FALSE)
                warning2 <- tkmessageBox(title="Danger of overfitting", 
                    message=paste("\tMaximum number of model df: ",
                    sum(allLevels), "\n\t\tTotal df available: ", (nSamples-1), 
                    "\n\nThere are not enough samples to fit all of the model ",
                    "terms\nand interactions. You may see overfitting of the ",
                    "data which\nmay cause problems with the results.\n\nPress ",
                    "'OK' to continue anyway, or press 'Cancel' to remove\n",
                    "more variables from the interactions list.",
                    sep=""), icon="warning", type="okcancel")
                if (as.character(warning2) == "cancel") {
                    tkfocus(tt2)
                    hgOff()
                    stop()
                } 
            }

            tcl("update", "idletasks")
            tkfocus(tt)
            tcl("update", "idletasks")
            tkdestroy(tt2)
            tcl("update", "idletasks")


            ###################################################################
            #                       9 - MODEL CALCULATION                     #
            ###################################################################

            ## Calculate optimum model for each gene by running a stepwise 
            ## GLM calculation first on the main effect variables, then on the
            ## first order of interactions of the significant main effects. 
            ## Uses a type II Anova calculation to test for significance p values    
            if (MEorINT == "ME") {
                tempP <- modelCalcME(adjParamData, d, startingModel, 
                    progress)
            } else if (MEorINT == "INT") {
                tempP <- modelCalcINT(adjParamData, d, param4INT, 
                    startingModel, progress)
            }
            pVal <- tempP$pValues
            failedAnova <- tempP$failedAnova
            errorGenes <- tempP$errorGenes
                
            ## Use multiple testing correction to limit false positives
            adjp <- mulTestCor(pVal, MTC, adjParamData, d, param4INT) 

            ###################################################################
            #                            10 - RESULTS                         #
            ###################################################################

            tkdestroy(tt)

            ## Output gene lists with associated p values. Also attached is the 
            ## gene list "ErrorGenes" which gives genes showing aliassing. This 
            ## can mess up the model calculations, so the user should be wary 
            ## about results for these genes
            
         if (MEorINT == "INT") {
             ## All possible interactions with the variables of interest
             outputNames <- gsub(" ", "_", names(adjParamData))
             if (length(param4INT) > 1) {
                 for (i in 1:length(param4INT)) {		
                     for (j in 1:length(param4INT)) {
                         if (param4INT[j] != param4INT[i]) {
                             outputNames <- c(outputNames, gsub(" ", "_", 
                                 paste(param4INT[i], ":", param4INT[j], sep="")))
                         }
                     }
                     rm(j)
                 }
                 rm(i)
             }
         } else outputNames <- gsub(" ", "_", names(adjParamData))
     
         tt3 <- tktoplevel()
         tktitle(tt3) <- "Envisage Output selection"
         mainFrame <- tkframe(tt3)
         fontLab <- tkfont.create(family="times",size=12,weight="bold")
     
         frameTitle <- tkframe(mainFrame)
         tkgrid(tklabel(frameTitle, text="Variable\nName", width=20, 
             font=fontLab), tklabel(frameTitle, text="All\nChanging\nGenes", 
             width=10, font=fontLab), tklabel(frameTitle,
             text="Significantly\nChanging\nGenes", width=10, font=fontLab))
         tkgrid(frameTitle,sticky="w")
     
         ## Scrollable frame for variable selection
         sw3 <- tkwidget(mainFrame,"ScrolledWindow",
             relief="sunken",borderwidth=2)
         sf2 <- tkwidget(sw3, "ScrollableFrame", width=470)
         tcl(sw3,"setwidget",sf2)
         subfID2 <- tclvalue(tcl(sf2,"getframe"))
         
         frameBox <- list()
         paramName <- list()
         geneChange <- list()
         sigGeneChange <- list()
         outputBox <- list()
         outBoxValue <- list()
         percentGenes <- NULL
         sigGenes <- NULL
     
         ## Order variables based on number of genes showing changes
         for (i in 1:length(outputNames)) {
             ## Count total number of genes changing for each variable
             temp1 <- 0
             for (j in 1:nrow(d)) {
                 if (!is.na(adjp[[outputNames[i]]][j])) { 
                     temp1 <- temp1 + 1
                 }
             }
             percent1 <- ((temp1/nrow(d))*100)
             percent1 <- as.numeric(substr(percent1,1,5))
             percentGenes <- c(percentGenes, percent1)
     
             ## Count number of significantly changing genes
             temp2 <- 0
             for (j in 1:nrow(d)) {
                 if (!is.na(adjp[[outputNames[i]]][j]) && 
                     adjp[[outputNames[i]]][j] <= pCutoff) {
                     temp2 <- temp2 + 1
                 }
             }
             percent2 <- ((temp2/nrow(d))*100)
             percent2 <- as.numeric(substr(percent2,1,5))
             sigGenes <- c(sigGenes, percent2)
         }
         rm(i)
     
         names(sigGenes) <- outputNames
         ordSigGenes <-  sort(sigGenes, decreasing=T)
         names(percentGenes) <- outputNames
         ordParInt <- percentGenes[names(ordSigGenes)] 
     
         ## Create scrolled frame with all variables and interactions
         for (i in 1:length(ordParInt)) 
         {
             frameBox[[i]] <- tcl("frame", paste(subfID2,".",i,sep=""), 
                 relief="flat", borderwidth=2)
     
             ## Variable name
             paramName[[i]] <- tcl("label", paste(subfID2,".",i,".1",sep=""), 
                 text=names(ordParInt)[i], width=30, justify="left")
     
             ## Percentage of genes showing changes for each variable and 
             ## interaction
             geneChange[[i]] <- tcl("label", paste(subfID2,".",i,".2",sep=""), 
                 text=paste(ordParInt[i],"%"), width=15)
     
             ## Percentage of genes showing significant changes for each 
             ## variable and interaction
             sigGeneChange[[i]] <- tcl("label", paste(subfID2,".",i,".3",sep=""), 
                 text=paste(ordSigGenes[i],"%"), width=15)
     
             ## Select output variables
             outputBox[[i]] <- tcl("checkbutton", paste(subfID2,
                 ".",i,".4",sep=""), width=10)
             tkconfigure(outputBox[[i]], cursor="hand2")
             outBoxValue[[i]] <- tclVar("1")
             tkconfigure(outputBox[[i]], variable=outBoxValue[[i]])
     
             ## Combine
             tkgrid(paramName[[i]], geneChange[[i]], sigGeneChange[[i]],
                 outputBox[[i]])
             tkgrid(frameBox[[i]])
         }
         rm(i)
         tkgrid(sw3)
                 
         OnOK2 <- function() {
             options(show.error.messages=TRUE)
             mainParams <- NULL
             for (i in 1:length(ordParInt)) {
                 if (tclvalue(outBoxValue[[i]]) == "1") {
                     mainParams <- c(mainParams,names(ordParInt)[i])
                 }
             }
             rm(i)
     
             ## Ensure at least one variable is selected 
             if (length(mainParams) == 0) {
                 warning <- tkmessageBox(message=paste("Please ensure",
                     "that at least one variable is selected"),
                     icon="warning", type="ok")
                 options(show.error.messages=FALSE)
                 stop()
             }
     
             tkdestroy(tt3)
     
             ## Pulls out genes whose p values are within the cutoff limit
             results <<- list()
             for (i in 1:length(mainParams)) {
                 tempP <- NULL
                 tempN <- NULL
                 for (j in 1:nrow(d)) {
                     if (!is.na(adjp[[mainParams[i]]][j])) { 
                         if (adjp[[mainParams[i]]][j] <= pCutoff) {	
                             tempP <- c(tempP, adjp[[mainParams[i]]][j])
                             tempN <- c(tempN, rownames(d)[j])
                         }
                     }
                 }
                 rm(j)
     	    
                 if (length(tempP) != 0) {
                     names(tempP) <- tempN
                     results[[i]] <- tempP
                 } else results[[i]] <- NA
                 names(results)[i] <- mainParams[i]
             }
             rm(i)
     
             ## Get rid of NA results
             list <- NULL
             for (i in 1:length(results)) {
                 if (!is.na(results[i])) list <- c(list,i)
             }
             rm(i)
             results <<- results[list]
     
             ## Create a directory in the working directory to store resulting
             ## gene lists
             try(dir.create(paste(fileResults, "/EnvisageResults", sep=""), 
                 showWarnings=F))
                 
             ## If results folder is non-empty, rename all old files
             dirs <- dir(paste(fileResults, "/EnvisageResults", sep=""))
             if (length(dirs) > 0) {
                 for (i in 1:length(dirs)) {
                     temp <- paste(substr(dirs[i],1,nchar(dirs[i])-4), "_OLD.txt", sep="")
                     file.rename(paste(fileResults, "/EnvisageResults/", dirs[i], sep=""),
                         paste(fileResults, "/EnvisageResults/", temp, sep=""))
                 }
             }         
     
             ## Write genelists to tab delimited text files in the new directory
             if (length(results) > 0) {
                 Pnames <- gsub(" ","_",names(results))
                 Pnames <- gsub(":","_INT_",Pnames)
                 for (i in 1:(length(results))) {
                     if (file.exists(paste(fileResults, "/EnvisageResults/", 
                         Pnames[i],".txt",sep=""))) {
                         file.remove(paste(fileResults, "/EnvisageResults/", 
                             Pnames[i],".txt",sep=""))
                     }
                     write.table(results[[i]],file=paste(fileResults,"/EnvisageResults/", 
                         Pnames[i],".txt",sep=""), append=F, 
                         row.names=T, col.names=FALSE, quote=FALSE, sep="\t")
                 }
                 rm(i)
             } else if (length(results) == 0) {
               warning <- tkmessageBox(message=paste("Warning: No genes found to ",
                   "be significant.", sep=""), icon="warning", type="ok") 
             }
     
             ## Output information for error genes
             list <- NULL
             for (i in 1:nrow(d)) {
                 if (!is.na(errorGenes[[i]][1])) list <- c(list, i)
             }
             rm(i)
     
             if (length(list) > 0) {
                 errorGenesShort <- errorGenes[list]
     
                 ## Write gene list for errors
                 if (file.exists(paste(fileResults, 
                     "/EnvisageResults/AliasGenes.txt", sep=""))) {
                     file.remove(paste(fileResults, 
                         "/EnvisageResults/AliasGenes.txt", sep=""))
                 }
                 write.table(names(errorGenesShort),file=paste(fileResults,
                     "/EnvisageResults/AliasGenes.txt", sep=""), append=F, 
                     row.names=F, col.names=FALSE, quote=FALSE, sep="\t")
     
                 ## Write error report
                 if (file.exists(paste(fileResults, 
                     "/EnvisageResults/AliasReport.txt", sep=""))) { 
                     file.remove(paste(fileResults, 
                         "/EnvisageResults/AliasReport.txt", sep=""))
                 }
                 for (i in 1:length(errorGenesShort)) {        
                     write(names(errorGenesShort)[i], file=paste(fileResults,
                         "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                     write("\nAliasing is seen across the following conditions:",
                         file=paste(fileResults,
                         "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                     for (j in 1:length(errorGenesShort[[i]])) {
                         write(errorGenesShort[[i]][j], file=paste(fileResults,
                             "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                     }
                     write("\n\n", file=paste(fileResults,
                         "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                 } 
             }
     
             ## Write gene list for genes that fail the Anova
             if (length(failedAnova) > 0) {
                 try(file.remove(paste(fileResults, 
                     "/EnvisageResults/Failed.txt", sep="")))
                 write.table(failedAnova,file=paste(fileResults,
                     "/EnvisageResults/Failed.txt", sep=""), append=F, 
                     row.names=F, col.names=FALSE, quote=FALSE, sep="\t")
             }
             
             ## Info on errors
             temp <- NULL
             tmpNames <- NULL
             for (i in 1:length(errorGenes)) {
                 if (!is.na(errorGenes[[i]][1])) {
                     temp <- c(temp, i)
                     tmpNames <- c(tmpNames, names(errorGenes)[i])
                 }
             }
             temp2 <- list()
             if (length(temp) > 0) {
                 for (i in 1:length(temp)) temp2[[i]] <- errorGenes[[temp[i]]]
                 names(temp2) <- names(errorGenes)[temp]
                 results[[length(results)+1]] <- temp2
                 names(results)[length(results)] <- "errorGenes"
             }
             if (length(failedAnova) > 0) {
                 results[[length(results)+1]] <- failedAnova
                 names(results)[length(results)] <- "failedAnova" 
             } 
         }       
     
         ## Function to select all check boxes
         checkAll <- function() {
             for (i in 1:length(ordParInt)) {    
                 outBoxValue[[i]] <- tclVar("1")
                 tkconfigure(outputBox[[i]], variable=outBoxValue[[i]])
             }
         }
     
         ## Function to clear all check boxes
         clearAll <- function() {
             for (i in 1:length(ordParInt)) {    
                 outBoxValue[[i]] <- tclVar("0")
                 tkconfigure(outputBox[[i]], variable=outBoxValue[[i]])
             }
         }
     
         ## Allow user to check or clear all boxes in one go
         frameSelect <- tkframe(mainFrame, relief="groove", borderwidth=2)
             checkAllBut <- tkbutton(frameSelect, text="Select All",
                 command=checkAll, bg="grey", cursor="hand2")
             clearAllBut <- tkbutton(frameSelect, text="Clear All",
                 command=clearAll, bg="grey", cursor="hand2")
             tkgrid(checkAllBut,clearAllBut)
         tkgrid(frameSelect)
     
         ## Select OK to save the results to the specified file
         frameOk <- tkframe(mainFrame, relief="flat", borderwidth=2)
             OK.but3 <- tkbutton(frameOk, text="   OK   ",
                 command=OnOK2, height=1, width=10, bg="grey", cursor="hand2")
         
             ## Return has the same function as OK
             tkbind(tt3,"<Return>",OnOK2)
             blankLine2 <- tklabel(frameOk,text=" ")
             tkgrid(blankLine2)
             tkgrid(OK.but3)
         tkgrid(frameOk)
         tkgrid(mainFrame) 
     
         ## Make sure that errors are shown if user quits by destroying the window
    tkbind(tt3,"<Destroy>",function() options(show.error.messages=TRUE))

        }
###############################################################################
##                                  End - OnOK2                              ##
###############################################################################     

        ## Function to select all check boxes
        checkAll2 <- function() {
            for (i in 1:length(adjParamData)) {    
                outBoxValue2[[i]] <<- tclVar("1")
                tkconfigure(outputBox2[[i]], variable=outBoxValue2[[i]])
            }
        }

        ## Function to clear all check boxes
        clearAll2 <- function() {
            for (i in 1:length(adjParamData)) {    
                outBoxValue2[[i]] <<- tclVar("0")
                tkconfigure(outputBox2[[i]], variable=outBoxValue2[[i]])
            }
        }

        OnCancel <- function() {
            ## Ensure error messages are switched back on (if recalled)
            options(show.error.messages=TRUE)
            tkfocus(tt)
            tkdestroy(tt2)
            hgOff()
        }

        ## Allow user to check or clear all boxes in one go
        frameSelect2 <- tkframe(mainFrame2, relief="groove", borderwidth=2)
            checkAllBut2 <- tkbutton(frameSelect2, text="Select All",
                command=checkAll2, bg="grey", cursor="hand2")
            clearAllBut2 <- tkbutton(frameSelect2, text="Clear All",
                command=clearAll2, bg="grey", cursor="hand2")
            tkgrid(checkAllBut2,clearAllBut2)
        tkgrid(frameSelect2)

        ## Explanation for selecting variables for interactions
        INTselectFrame <- tkframe(mainFrame2, borderwidth=2)
            INTselectLabel <- tklabel(INTselectFrame, text=paste("Please choose",
                " the variables that you would like to look at the\n",
                "interactions for. Please note that looking at interactions ",
                "increases\nthe number of terms in the model and may produce ",
                "overfitting if too\nmany are selected.", sep=""), 
                justify="left")
            tkgrid(INTselectLabel)
        tkgrid(INTselectFrame)

        ## Select OK to run function
        frameOk <- tkframe(mainFrame2, relief="flat", borderwidth=2)
            OK.but2 <- tkbutton(frameOk, text="   OK   ",
                command=OnOK2, height=1, width=10, bg="grey", cursor="hand2")
            Cancel.but2 <- tkbutton(frameOk, text=" Cancel ",
                command=OnCancel, height=1, width=10, bg="grey", cursor="hand2")
            ## Return has the same function as OK
            tkbind(tt2,"<Return>",OnOK2)
            tkgrid(OK.but2, Cancel.but2)
        tkgrid(frameOk)
        tkgrid(mainFrame2) 

        destroyTT2 <- function()
        {
            options(show.error.messages=TRUE)
            hgOff()
        }

        ## Make sure that errors are shown if user quits by destroying the 
        tkbind(tt2,"<Destroy>",destroyTT2())
        tkfocus(tt2)
        tcl("update", "idletasks")
    }
    

    ###########################################################################
    #                     B - ME ONLY MODEL CALCULATION                       #
    ###########################################################################

    ## Calculate optimum model for each gene by running a stepwise 
    ## GLM calculation first on the main effect variables, then on the
    ## first order of interactions of the significant main effects
    if (MEorINT == "ME") {
        tempP <- modelCalcME(adjParamData, d, startingModel,
            progress)
            pVal <- tempP$pValues
            failedAnova <- tempP$failedAnova
            errorGenes <- tempP$errorGenes
                
        ## Use multiple testing correction to limit false positives
        adjp <- mulTestCor(pVal, MTC, adjParamData, d, param4INT) 

        #######################################################################
        #                               10 - RESULTS                          #
        #######################################################################

        tkdestroy(tt)

        ## Output gene lists with associated p values. Also attached is the gene 
        ## list "ErrorGenes" which gives genes showing aliassing. This can mess 
        ## up the model calculations, so the user should be wary about results 
        ## for these genes
         if (MEorINT == "INT") {
             ## All possible interactions with the variables of interest
             outputNames <- gsub(" ", "_", names(adjParamData))
             if (length(param4INT) > 1) {
                 for (i in 1:length(param4INT)) {		
                     for (j in 1:length(param4INT)) {
                         if (param4INT[j] != param4INT[i]) {
                             outputNames <- c(outputNames, gsub(" ", "_", 
                                 paste(param4INT[i], ":", param4INT[j], sep="")))
                         }
                     }
                     rm(j)
                 }
                 rm(i)
             }
         } else outputNames <- gsub(" ", "_", names(adjParamData))
     
         tt3 <- tktoplevel()
         tktitle(tt3) <- "Envisage Output selection"
         mainFrame <- tkframe(tt3)
         fontLab <- tkfont.create(family="times",size=12,weight="bold")
     
         frameTitle <- tkframe(mainFrame)
         tkgrid(tklabel(frameTitle, text="Variable\nName", width=20, 
             font=fontLab), tklabel(frameTitle, text="All\nChanging\nGenes", 
             width=10, font=fontLab), tklabel(frameTitle,
             text="Significantly\nChanging\nGenes", width=10, font=fontLab))
         tkgrid(frameTitle,sticky="w")
     
         ## Scrollable frame for variable selection
         sw3 <- tkwidget(mainFrame,"ScrolledWindow",
             relief="sunken",borderwidth=2)
         sf2 <- tkwidget(sw3, "ScrollableFrame", width=470)
         tcl(sw3,"setwidget",sf2)
         subfID2 <- tclvalue(tcl(sf2,"getframe"))
         
         frameBox <- list()
         paramName <- list()
         geneChange <- list()
         sigGeneChange <- list()
         outputBox <- list()
         outBoxValue <- list()
         percentGenes <- NULL
         sigGenes <- NULL
     
         ## Order variables based on number of genes showing changes
         for (i in 1:length(outputNames)) {
             ## Count total number of genes changing for each variable
             temp1 <- 0
             for (j in 1:nrow(d)) {
                 if (!is.na(adjp[[outputNames[i]]][j])) { 
                     temp1 <- temp1 + 1
                 }
             }
             percent1 <- ((temp1/nrow(d))*100)
             percent1 <- as.numeric(substr(percent1,1,5))
             percentGenes <- c(percentGenes, percent1)
     
             ## Count number of significantly changing genes
             temp2 <- 0
             for (j in 1:nrow(d)) {
                 if (!is.na(adjp[[outputNames[i]]][j]) && 
                     adjp[[outputNames[i]]][j] <= pCutoff) {
                     temp2 <- temp2 + 1
                 }
             }
             percent2 <- ((temp2/nrow(d))*100)
             percent2 <- as.numeric(substr(percent2,1,5))
             sigGenes <- c(sigGenes, percent2)
         }
         rm(i)
     
         names(sigGenes) <- outputNames
         ordSigGenes <-  sort(sigGenes, decreasing=T)
         names(percentGenes) <- outputNames
         ordParInt <- percentGenes[names(ordSigGenes)] 
     
         ## Create scrolled frame with all variables and interactions
         for (i in 1:length(ordParInt)) 
         {
             frameBox[[i]] <- tcl("frame", paste(subfID2,".",i,sep=""), 
                 relief="flat", borderwidth=2)
     
             ## Variable name
             paramName[[i]] <- tcl("label", paste(subfID2,".",i,".1",sep=""), 
                 text=names(ordParInt)[i], width=30, justify="left")
     
             ## Percentage of genes showing changes for each variable and 
             ## interaction
             geneChange[[i]] <- tcl("label", paste(subfID2,".",i,".2",sep=""), 
                 text=paste(ordParInt[i],"%"), width=15)
     
             ## Percentage of genes showing significant changes for each 
             ## variable and interaction
             sigGeneChange[[i]] <- tcl("label", paste(subfID2,".",i,".3",sep=""), 
                 text=paste(ordSigGenes[i],"%"), width=15)
     
             ## Select output variables
             outputBox[[i]] <- tcl("checkbutton", paste(subfID2,
                 ".",i,".4",sep=""), width=10)
             tkconfigure(outputBox[[i]], cursor="hand2")
             outBoxValue[[i]] <- tclVar("1")
             tkconfigure(outputBox[[i]], variable=outBoxValue[[i]])
     
             ## Combine
             tkgrid(paramName[[i]], geneChange[[i]], sigGeneChange[[i]],
                 outputBox[[i]])
             tkgrid(frameBox[[i]])
         }
         rm(i)
         tkgrid(sw3)
                 
         OnOK2 <- function() {
             options(show.error.messages=TRUE)
             mainParams <- NULL
             for (i in 1:length(ordParInt)) {
                 if (tclvalue(outBoxValue[[i]]) == "1") {
                     mainParams <- c(mainParams,names(ordParInt)[i])
                 }
             }
             rm(i)
     
             ## Ensure at least one variable is selected 
             if (length(mainParams) == 0) {
                 warning <- tkmessageBox(message=paste("Please ensure",
                     "that at least one variable is selected"),
                     icon="warning", type="ok")
                 options(show.error.messages=FALSE)
                 stop()
             }
     
             tkdestroy(tt3)
     
             ## Pulls out genes whose p values are within the cutoff limit
             results <<- list()
             for (i in 1:length(mainParams)) {
                 tempP <- NULL
                 tempN <- NULL
                 for (j in 1:nrow(d)) {
                     if (!is.na(adjp[[mainParams[i]]][j])) { 
                         if (adjp[[mainParams[i]]][j] <= pCutoff) {	
                             tempP <- c(tempP, adjp[[mainParams[i]]][j])
                             tempN <- c(tempN, rownames(d)[j])
                         }
                     }
                 }
                 rm(j)
     	    
                 if (length(tempP) != 0) {
                     names(tempP) <- tempN
                     results[[i]] <- tempP
                 } else results[[i]] <- NA
                 names(results)[i] <- mainParams[i]
             }
             rm(i)
     
             ## Get rid of NA results
             list <- NULL
             for (i in 1:length(results)) {
                 if (!is.na(results[i])) list <- c(list,i)
             }
             rm(i)
             results <<- results[list]
     
             ## Create a directory in the working directory to store resulting
             ## gene lists
             try(dir.create(paste(fileResults, "/EnvisageResults", sep=""), 
                 showWarnings=F))
                 
             ## If results folder is non-empty, rename all old files
             dirs <- dir(paste(fileResults, "/EnvisageResults", sep=""))
             if (length(dirs) > 0) {
                 for (i in 1:length(dirs)) {
                     temp <- paste(substr(dirs[i],1,nchar(dirs[i])-4), "_OLD.txt", sep="")
                     file.rename(paste(fileResults, "/EnvisageResults/", dirs[i], sep=""),
                         paste(fileResults, "/EnvisageResults/", temp, sep=""))
                 }
             }         
     
             ## Write genelists to tab delimited text files in the new directory
             if (length(results) > 0) {
                 Pnames <- gsub(" ","_",names(results))
                 Pnames <- gsub(":","_INT_",Pnames)
                 for (i in 1:(length(results))) {
                     if (file.exists(paste(fileResults, "/EnvisageResults/", 
                         Pnames[i],".txt",sep=""))) {
                         file.remove(paste(fileResults, "/EnvisageResults/", 
                             Pnames[i],".txt",sep=""))
                     }
                     write.table(results[[i]],file=paste(fileResults,"/EnvisageResults/", 
                         Pnames[i],".txt",sep=""), append=F, 
                         row.names=T, col.names=FALSE, quote=FALSE, sep="\t")
                 }
                 rm(i)
             } else if (length(results) == 0) {
               warning <- tkmessageBox(message=paste("Warning: No genes found to ",
                   "be significant.", sep=""), icon="warning", type="ok") 
             }
     
             ## Output information for error genes
             list <- NULL
             for (i in 1:nrow(d)) {
                 if (!is.na(errorGenes[[i]][1])) list <- c(list, i)
             }
             rm(i)
     
             if (length(list) > 0) {
                 errorGenesShort <- errorGenes[list]
     
                 ## Write gene list for errors
                 if (file.exists(paste(fileResults, 
                     "/EnvisageResults/AliasGenes.txt", sep=""))) {
                     file.remove(paste(fileResults, 
                         "/EnvisageResults/AliasGenes.txt", sep=""))
                 }
                 write.table(names(errorGenesShort),file=paste(fileResults,
                     "/EnvisageResults/AliasGenes.txt", sep=""), append=F, 
                     row.names=F, col.names=FALSE, quote=FALSE, sep="\t")
     
                 ## Write error report
                 if (file.exists(paste(fileResults, 
                     "/EnvisageResults/AliasReport.txt", sep=""))) { 
                     file.remove(paste(fileResults, 
                         "/EnvisageResults/AliasReport.txt", sep=""))
                 }
                 for (i in 1:length(errorGenesShort)) {        
                     write(names(errorGenesShort)[i], file=paste(fileResults,
                         "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                     write("\nAliasing is seen across the following conditions:",
                         file=paste(fileResults,
                         "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                     for (j in 1:length(errorGenesShort[[i]])) {
                         write(errorGenesShort[[i]][j], file=paste(fileResults,
                             "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                     }
                     write("\n\n", file=paste(fileResults,
                         "/EnvisageResults/AliasReport.txt", sep=""), append=T)
                 } 
             }
     
             ## Write gene list for genes that fail the Anova
             if (length(failedAnova) > 0) {
                 try(file.remove(paste(fileResults, 
                     "/EnvisageResults/Failed.txt", sep="")))
                 write.table(failedAnova,file=paste(fileResults,
                     "/EnvisageResults/Failed.txt", sep=""), append=F, 
                     row.names=F, col.names=FALSE, quote=FALSE, sep="\t")
             }
             
             ## Info on errors
             temp <- NULL
             tmpNames <- NULL
             for (i in 1:length(errorGenes)) {
                 if (!is.na(errorGenes[[i]][1])) {
                     temp <- c(temp, i)
                     tmpNames <- c(tmpNames, names(errorGenes)[i])
                 }
             }
             temp2 <- list()
             if (length(temp) > 0) {
                 for (i in 1:length(temp)) temp2[[i]] <- errorGenes[[temp[i]]]
                 names(temp2) <- names(errorGenes)[temp]
                 results[[length(results)+1]] <- temp2
                 names(results)[length(results)] <- "errorGenes"
             }
             if (length(failedAnova) > 0) {
                 results[[length(results)+1]] <- failedAnova
                 names(results)[length(results)] <- "failedAnova" 
             } 
         }       
     
         ## Function to select all check boxes
         checkAll <- function() {
             for (i in 1:length(ordParInt)) {    
                 outBoxValue[[i]] <- tclVar("1")
                 tkconfigure(outputBox[[i]], variable=outBoxValue[[i]])
             }
         }
     
         ## Function to clear all check boxes
         clearAll <- function() {
             for (i in 1:length(ordParInt)) {    
                 outBoxValue[[i]] <- tclVar("0")
                 tkconfigure(outputBox[[i]], variable=outBoxValue[[i]])
             }
         }
     
         ## Allow user to check or clear all boxes in one go
         frameSelect <- tkframe(mainFrame, relief="groove", borderwidth=2)
             checkAllBut <- tkbutton(frameSelect, text="Select All",
                 command=checkAll, bg="grey", cursor="hand2")
             clearAllBut <- tkbutton(frameSelect, text="Clear All",
                 command=clearAll, bg="grey", cursor="hand2")
             tkgrid(checkAllBut,clearAllBut)
         tkgrid(frameSelect)
     
         ## Select OK to save the results to the specified file
         frameOk <- tkframe(mainFrame, relief="flat", borderwidth=2)
             OK.but3 <- tkbutton(frameOk, text="   OK   ",
                 command=OnOK2, height=1, width=10, bg="grey", cursor="hand2")
         
             ## Return has the same function as OK
             tkbind(tt3,"<Return>",OnOK2)
             blankLine2 <- tklabel(frameOk,text=" ")
             tkgrid(blankLine2)
             tkgrid(OK.but3)
         tkgrid(frameOk)
         tkgrid(mainFrame) 
     
         ## Make sure that errors are shown if user quits by destroying the window
    tkbind(tt3,"<Destroy>",function() options(show.error.messages=TRUE))

    }
    } else hgOff()
}

###############################################################################
###############################################################################
##                                                                           ##
##                                 End - OnOK                                ##
##                                                                           ##
###############################################################################
###############################################################################



###############################################################################
###############################################################################
##                                                                           ##
##                             Function - OnCancel                           ##
##                                                                           ##
###############################################################################
###############################################################################

OnCancel <- function()
{
    ## Ensure error messages are switched back on (if recalled)
    options(show.error.messages=TRUE)

    ## Check that user wishes to quit
    quit <- tkmessageBox(title="Quit?", message=paste("This will end your ",
        "Envisage session.\nAre you sure you want to quit?", sep=""),
        icon="question",type="yesno",default="no")
    if (tclvalue(quit) == "yes") {
        tkdestroy(tt)	
    }
    if (tclvalue(quit) == "no") {
        tkfocus(tt)
    }
}

###############################################################################
###############################################################################
##                                                                           ##
##                               End - OnCancel                              ##
##                                                                           ##
###############################################################################
###############################################################################


## Turn all cursors into hourglass cursors
hgOn <- function() {
    ## Ensure error messages are switched back on (if recalled)
    options(show.error.messages=TRUE)

    ## Hourglass cursor while model calculation occurs
    tkconfigure(tt, cursor="watch")
    tkconfigure(pEntry,cursor="watch")
    tkconfigure(mtcSelect,cursor="watch")
    tkconfigure(fileInput,cursor="watch")
    tkconfigure(browseBut,cursor="watch")
    tkconfigure(radME,cursor="watch")
    tkconfigure(radINT,cursor="watch")
    tkconfigure(Mentry,cursor="watch")
    tkconfigure(resetBut,cursor="watch")
    for (i in 1:nParam) {
        tkconfigure(radSel[[i]],cursor="watch")
        tkconfigure(radNum[[i]],cursor="watch")  
        tkconfigure(radCat[[i]],cursor="watch")  
    }     
    tkconfigure(OK.but, cursor="watch")
    tkconfigure(Cancel.but, cursor="watch")
}

## Turn all cursors back to their default
hgOff <- function() 
{
    tkconfigure(tt,cursor="arrow")
    tkconfigure(pEntry, cursor="ibeam")
    tkconfigure(mtcSelect,cursor="hand2")
    tkconfigure(fileInput,cursor="ibeam")
    tkconfigure(browseBut,cursor="hand2")
    tkconfigure(radME,cursor="hand2")
    tkconfigure(radINT,cursor="hand2")
    tkconfigure(Mentry,cursor="ibeam")
    tkconfigure(resetBut,cursor="hand2")
    for (i in 1:nParam) {
        tkconfigure(radSel[[i]],cursor="hand2")
        tkconfigure(radNum[[i]],cursor="hand2")  
        tkconfigure(radCat[[i]],cursor="hand2")  
    }  
    tkconfigure(OK.but, cursor="hand2")
    tkconfigure(Cancel.but, cursor="hand2")
    options(show.error.messages=FALSE)    
    stop()
}

frameOkCancel <- tkframe(frameOverall,relief="flat",borderwidth=2)

    OK.but <- tkbutton(frameOkCancel, text="   OK   ",
        command=OnOK, height=1, width=10, bg="grey",cursor="hand2")
    Cancel.but <- tkbutton(frameOkCancel, text=" Cancel ",
        command=OnCancel, height=1, width=10, bg="grey",cursor="hand2")
    
    ## Enter and Escape have the same function as OK and Cancel
    tkbind(tt,"<Return>",OnOK)
    tkbind(tt,"<Escape>",OnCancel)
    blankLine <- tklabel(frameOkCancel,text=" ")
    tkgrid(blankLine)
    tkgrid(OK.but,Cancel.but)
    tkgrid(blankLine,sticky="ns")

tkgrid(frameOkCancel)
tkgrid.configure(frameOkCancel,columnspan=2)
tkgrid(frameOverall)

## Make sure that errors are shown if user quits by destroying the 
tkbind(tt,"<Destroy>",function() options(show.error.messages=TRUE))

## Ensure window is correctly loaded when the window is moved
tkbind(tt,"<Configure>",function() tcl("update", "idletasks"))

## Change cursors for all widgets in GUI
hgOff()

## Focus on the main window
tkfocus(tt)


} else if (widget==FALSE) {
    ###########################################################################
    #                        4 - CHECK ARGUMENT VALUES                        #
    ###########################################################################
   
    ## Check MTC
    mtcShort <- c("None", "BH", "BY", "Bonferroni", "Holm", "Hochberg",
        "SidakSS", "SidakSD") 
    if (!MTC %in% mtcShort) stop(paste("Selected multiple testing correction", 
        "not recognised.\nPlease choose from 'None', 'BH',", 
        "'BY', 'Bonferroni', 'Holm', 'Hochberg',\n'SidakSS'",
        "or 'SidakSD'"))
   
    ## Check pCutoff
    if (pCutoff < 0 || pCutoff > 1) stop("P value must be between 0 and 1")

    ## Check MEorINT
    if (!MEorINT %in% c("ME", "INT")) stop("MEorINT must be either 'ME' or 'INT'")
   
    ## Check useParams and make sure the names match
    for (i in 1:length(useParams)) {
        if (substr(useParams[i], 1, 1) %in% 0:9) {
            useParams[i] <- paste("X", useParams[i], sep="")
        }
    }
    rm(i)
    useParams <- gsub("-", "_", useParams)
    useParams <- gsub("/", "_", useParams)
    useParams <- gsub(" ", "_", useParams)

    
    if (length(useParams) == 0) {
        stop(paste("Please ensure that at least one variable is selected",
            "\nin useParams"))
    }

    for (i in 1:length(useParams)) {
        if (!useParams[i] %in% paramNames) stop(paste("Please ensure that ",
            "the variable name '", useParams[i], "'\nin useParams matches the ",
            "variable names in your ExpressionSet", sep=""))
    }
    
    ## Check paramType
    if (length(paramType) != length(useParams)) {
        stop("Check that length of 'paramType' matches 'useParams'")
    }
    
    for (i in 1:length(paramType)) {
        if (!paramType[i] %in% c("numeric", "categoric")) {
            stop(paste("paramType '", paramType[i], "' not recognised. ",
                "Please choose from\n'numeric' and 'categoric'", sep=""))
        }
    }

    ## Check fileResults
    if (file.access(fileResults, mode=0) != 0) stop("Specified file not found")
    
    ## Check param4INT
    if (length(param4INT) > length(useParams)) {
        stop("More variables selected for interactions than main effects")
    }
    
    ## Check param4INT
    for (i in 1:length(param4INT)) {
        if (substr(param4INT[i], 1, 1) %in% 0:9) {
            param4INT[i] <- paste("X", param4INT[i], sep="")
        }
    }
    rm(i)
    param4INT <- gsub("-", "_", param4INT)
    param4INT <- gsub("/", "_", param4INT)
    param4INT <- gsub(" ", "_", param4INT)
    
    for (i in 1:length(param4INT)) {
        if (!param4INT[i] %in% useParams) stop(paste("Interaction variable", 
        param4INT[i], "not included\nin useParams"))
    }
    
    ## Check startModel
    if (length(startModel) > 0) {
        temp <- modelEntry(glm.formula=startModel)
        startingModel <- temp[[1]]
        modelTerms <- temp[[2]]
        intPos <- temp[[3]]
    
        if (length(intPos) > 0 && MEorINT == "ME") {
            stop(paste("Specified model contains interactions, but you have",
                "chosen to only model main effect terms."))
        }
    
        for (i in 1:length(modelTerms)) {
            if (!gsub("_", " ", modelTerms[i]) %in% useParams) {
                stop(paste("Please ensure that all terms in the selected model",
                    "are present in useParams"))
            }
        }
    } else startingModel <- NULL
    
    ## Ensure that highly correlated variables are not selected
    if (length(corParams) != 0) {
        for (i in 1:length(corParams)) {
            count <- 0
            for (j in 1:length(corParams[[i]])) {
                if (startParamNames[corParams[[i]][j]] %in% useParams) {
                    count <- count +1
                }
            }
            if (count > 1) {
                message <- c("Warning: The variables:\n\n")
                for (j in 1:length(corParams[[i]])) {
                    if (startParamNames[corParams[[i]][j]] %in% useParams) {
                        message <- paste(message, "\t", 
                            startParamNames[corParams[[i]][j]], "\n", sep="")
                    }
                }
                rm(j)
                message <- paste(message, "\nshow very high ",
                    "correlation. Please ensure that only\none of ",
                    "these is selected", sep="")           
                stop(message)
            }
        }
    }
    
    
    ###########################################################################
    #                     5 - ASSIGN CLASSES TO VARIABLES                     #
    ###########################################################################

    ## Ensure that numerical variables are not stored as factors in the design 
    ## matrix
    adjParamData <- paramData[,useParams]
    for (i in 1:length(useParams)) {
        if (paramType[i] == "numeric") {
            adjParamData[[i]] <- 
                as.numeric(as.character(paramData[[useParams[i]]]))
        }
    }


    ###########################################################################
    #         6 - ENSURE THAT LABELLING PARAMETERS ARE REMOVED                #
    ###########################################################################

    ## Ensure that no labelling variables are selected
    if (length(labParams) != 0) {
        warnLabParams <- NULL
        for (i in 1:length(labParams)) {
            if (labParams[i] %in% useParams) { 
                if (class(adjParamData[[labParams[i]]]) == "factor") {
                    warnLabParams <- c(warnLabParams, labParams[i])
                }
            }
        }
        rm(i)

        ## Warn user about labelling variables
        if (length(warnLabParams) != 0) {
            message <- "The variables "

            ## List possible labelling variables
            for (i in 1:length(warnLabParams)) {
                message <- paste(message, "'", warnLabParams[i],
                    "', ", sep="")
            }
            rm(i)

            ## Warning message
            message <- paste(message,"\nhave unique values for each sample.",
                " Either ensure they are selected as\nnumeric in paramType, ",
                "or do not use (to avoid overfitting)", sep="")           
            stop(message)
        }
    }


    ## Extract updated variable information
    paramNames <- useParams
    nParam <- length(paramNames)

    ## If only one variable chosen, treat as only main effect
    if (nParam == 1) MEorINT <- "ME"   
    if (length(param4INT) == 0) MEorINT <- "ME"


    if (MEorINT == "INT") {
        ###################################################################
        #                       9 - MODEL CALCULATION                     #
        ###################################################################

        ## Calculate optimum model for each gene by running a stepwise 
        ## GLM calculation first on the main effect variables, then on the
        ## first order of interactions of the significant main effects. 
        ## Uses a type II Anova calculation to test for significance p values    
        if (MEorINT == "ME") {
            tempP <- modelCalcME(adjParamData, d, startingModel)
        } else if (MEorINT == "INT") {
            tempP <- modelCalcINT(adjParamData, d, param4INT, 
                startingModel)
        }
        pVal <- tempP$pValues
        failedAnova <- tempP$failedAnova
        errorGenes <- tempP$errorGenes
                 
        ## Use multiple testing correction to limit false positives
        adjp <- mulTestCor(pVal, MTC, adjParamData, d, param4INT) 

        ###################################################################
        #                            10 - RESULTS                         #
        ###################################################################
    
        ## Output gene lists with associated p values. Also attached is the 
        ## gene list "ErrorGenes" which gives genes showing aliassing. This 
        ## can mess up the model calculations, so the user should be wary 
        ## about results for these genes
        results <- list()
        results <- result.output(adjp, pCutoff, adjParamData, d, errorGenes, 
            fileResults, MEorINT, param4INT, failedAnova)

        ## Info on errors
        temp <- NULL
        tmpNames <- NULL
        for (i in 1:length(errorGenes)) {
            if (!is.na(errorGenes[[i]][1])) {
                temp <- c(temp, i)
                tmpNames <- c(tmpNames, names(errorGenes)[i])
            }
        }
        temp2 <- list()
        if (length(temp) > 0) {
            for (i in 1:length(temp)) temp2[[i]] <- errorGenes[[temp[i]]]
            names(temp2) <- names(errorGenes)[temp]
            results[[length(results)+1]] <- temp2
            names(results)[length(results)] <- "errorGenes"
        }
        if (length(failedAnova) > 0) {
            results[[length(results)+1]] <- failedAnova
            names(results)[length(results)] <- "failedAnova" 
        }              
            
     } else if (MEorINT == "ME") {
        tempP <- modelCalcME(adjParamData, d, startingModel)
            pVal <- tempP$pValues
            failedAnova <- tempP$failedAnova
            errorGenes <- tempP$errorGenes
                
        ## Use multiple testing correction to limit false positives
        adjp <- mulTestCor(pVal, MTC, adjParamData, d, param4INT) 

        #######################################################################
        #                               10 - RESULTS                          #
        #######################################################################

        ## Output gene lists with associated p values. Also attached is the gene 
        ## list "ErrorGenes" which gives genes showing aliassing. This can mess 
        ## up the model calculations, so the user should be wary about results 
        ## for these genes
        results <- list()
        results <- result.output(adjp, pCutoff, adjParamData, d, errorGenes, 
            fileResults, MEorINT, param4INT, failedAnova)

        ## Info on errors
        temp <- NULL
        tmpNames <- NULL
        for (i in 1:length(errorGenes)) {
            if (!is.na(errorGenes[[i]][1])) {
                temp <- c(temp, i)
                tmpNames <- c(tmpNames, names(errorGenes)[i])
            }
        }
        temp2 <- list()
        if (length(temp) > 0) {
            for (i in 1:length(temp)) temp2[[i]] <- errorGenes[[temp[i]]]
            names(temp2) <- names(errorGenes)[temp]
            results[[length(results)+1]] <- temp2
            names(results)[length(results)] <- "errorGenes"
        }
        if (length(failedAnova) > 0) {
            results[[length(results)+1]] <- failedAnova
            names(results)[length(results)] <- "failedAnova" 
        }            
    }
    return(results)
}
}
###############################################################################
###############################################################################
##                        END - FUNCTION - envisage                          ##
###############################################################################
###############################################################################
