## Copyright (C) 2007 Sam Robson
##
## This file is part of Envisage.
##
## Envisage is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Envisage is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Envisage; if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

###############################################################################
#                        BEGIN - FUNCTION - modelCalcINT                      #
###############################################################################

## Function that calculates main effect linear models using a step-wise 
## iterative process, calculates first order interaction models if necessary, 
## then uses a type II Anova calculation to test for significance

modelCalcINT <- function(paramData, exprData, param4INT, startingModel, 
    progress) 
{   
    errorGenes <- list()
    p <- list()    
    failedAnova <- NULL
    if (!missing(progress)) tcl("update", "idletasks")

    for (i in 1:nrow(exprData)) {
        ## Model calculation - First order interactions
        model <- NULL
        allData <- cbind(exprData[i,], paramData)
        colnames(allData) <- c("x", gsub(" ", "_", names(paramData)))

        ## If all variables are to be used for interactions
        if (length(param4INT) == length(paramData)) {
            genLin <- glm(x ~ .^2, data=allData, na.action=na.omit, 
                family=gaussian)
            stepGenLin <- step(genLin, scope=list(lower=startingModel, 
                upper="x ~ .^2"), direction="backward", trace=0)
            model <- stepGenLin
        } else if (length(param4INT) != length(paramData)) {
            ## If only some of the variables are to be used for 
            ## interactions
            MEterms <- NULL
            INTterms <- NULL
            for (j in 1:length(paramData)) {
                if (names(paramData)[j] %in% param4INT) {
                    INTterms <- c(INTterms, names(paramData)[j])
                } else MEterms <- c(MEterms, names(paramData)[j])
            }

            ## Build main effect terms into the formula
            formulaMain <- "x ~ "
            if (length(MEterms) > 0) {
                for (j in 1:length(MEterms)) {
                    formulaMain <- paste(formulaMain, gsub(" ", "_", 
                        MEterms[j]), " + ", sep="")
                }
                rm(j)
            }
   
            ## Build terms into formula for interactions
            if (length(INTterms) > 0) {
                formulaMain <- paste(formulaMain, "(", sep="")
                for (j in 1:length(INTterms)) {
                    formulaMain <- paste(formulaMain, gsub(" ", "_", 
                        INTterms[j]), " + ", sep="")
                }
                rm(j)
            }
            formulaMain <- substr(formulaMain, 1, (nchar(formulaMain)-3))
            if (length(INTterms) > 0) formulaMain <- paste(formulaMain, 
                ")^2", sep="")
            formulaMain <- as.formula(formulaMain)

            ## Stepwise modelling
            genLin <- glm(formulaMain, data=allData, na.action=na.omit, 
                family=gaussian)
            stepGenLin <- step(genLin, scope=list(lower=startingModel, 
                upper=formulaMain), direction="backward", trace=0)
            model <- stepGenLin
        }

        ## Information about aliasing
        error <- NULL
        temp <- alias(model)$Complete
        if (length(temp) > 0) {
            for (j in 1:nrow(temp)) {
                for (k in 1:ncol(temp)) {
                    if (abs(temp[j,k]) > 0) {
                        error <- c(error, paste(rownames(temp)[j], " & ",
                            colnames(temp)[k], sep=""))
                    }
                }
            }
            if (length(error) > 0) {
                errorGenes[[i]] <- error
                class(model) <- c("glmNA", "glm", "lm")
            } else if (length(error) == 0) {
                errorGenes[[i]] <- NA
                class(model) <- c("glmNA", "glm", "lm")
            }
        } else errorGenes[[i]] <- NA
        names(errorGenes)[i] <- rownames(exprData)[i]

        ## Uses a type II anova method to calculate p values for each variable 
        ## and interaction for each gene
        anovaResults <- NULL
        if (length(model$coefficients) > 1) {
            anovaResults <- Anova(model,type="II",test="F")
            temp <- anovaResults$'Pr(>F)'
            if (!all(is.na(temp))) {
                p[[i]] <- temp[1:length(temp)-1]
                names(p[[i]]) <- rownames(anovaResults)[1:length(temp)-1]
            } else if (all(is.na(temp))) {
                p[[i]] <- NA
                failedAnova <- c(failedAnova, rownames(exprData)[i])
            }
        } else p[[i]] <- NA
	
        ## Update progress report
        if (!missing(progress)) {
            tclvalue(progress) <- i
            tcl("update", "idletasks")
        }	
    }
    rm(i)
    names(p) <- rownames(exprData)
    out <- list(pValues=p, failedAnova=failedAnova, errorGenes=errorGenes)
    return(out)
}

###############################################################################
#                          END - FUNCTION - modelCalcINT                      #
###############################################################################