## Copyright (C) 2007 Sam Robson
##
## This file is part of Envisage.
##
## Envisage is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Envisage is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Envisage; if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

###############################################################################
#                      BEGIN - FUNCTION - result.output                       #
###############################################################################

## Function that takes the corrected p-Values from the model calculations to 
## discern genes that change significantly across the variables of interest,
## then outputs them to the user

result.output <- function(adjp, pCutoff, pData, exprData, errorGenes, 
    fileResults, MEorINT, param4INT, failedAnova) 
{
    if (MEorINT == "INT") {
        ## All possible interactions with the variables of interest
        outputNames <- gsub(" ", "_", names(pData))
        if (length(param4INT) > 1) {
            for (i in 1:length(param4INT)) {		
                for (j in 1:length(param4INT)) {
                    if (param4INT[j] != param4INT[i]) {
                        outputNames <- c(outputNames, gsub(" ", "_", 
                            paste(param4INT[i], ":", param4INT[j], sep="")))
                    }
                }
                rm(j)
            }
            rm(i)
        }
    } else outputNames <- gsub(" ", "_", names(pData))

    ## Pulls out genes whose p values are within the cutoff limit
    results <- list()
    for (i in 1:length(outputNames)) {
        tempP <- NULL
        tempN <- NULL
        for (j in 1:nrow(exprData)) {
            if (!is.na(adjp[[outputNames[i]]][j])) { 
                if (adjp[[outputNames[i]]][j] <= pCutoff) {	
                    tempP <- c(tempP, adjp[[outputNames[i]]][j])
                    tempN <- c(tempN, rownames(exprData)[j])
                }
            }
        }
        rm(j)
	    
        if (length(tempP) != 0) {
            names(tempP) <- tempN
            results[[i]] <- tempP
        } else results[[i]] <- NA
        names(results)[i] <- outputNames[i]
    }
    rm(i)

    ## Get rid of NA results
    list <- NULL
    for (i in 1:length(results)) {
        if (!is.na(results[i])) list <- c(list,i)
    }
    rm(i)
    results <- results[list]

    ## Create a directory in the working directory to store resulting
    ## gene lists
    try(dir.create(paste(fileResults, "/EnvisageResults", sep=""), 
        showWarnings=F))
    
    ## If results folder is non-empty, rename all old files
    dirs <- dir(paste(fileResults, "/EnvisageResults", sep=""))
    if (length(dirs) > 0) {
        for (i in 1:length(dirs)) {
            temp <- paste(substr(dirs[i],1,nchar(dirs[i])-4), "_OLD.txt", sep="")
            file.rename(paste(fileResults, "/EnvisageResults/", dirs[i], sep=""),
                paste(fileResults, "/EnvisageResults/", temp, sep=""))
        }
    }    

    ## Write genelists to tab delimited text files in the new directory
    if (length(results) > 0) {
        Pnames <- gsub(" ","_",names(results))
        Pnames <- gsub(":","_INT_",Pnames)
        for (i in 1:(length(results))) {
            if (file.exists(paste(fileResults, "/EnvisageResults/", 
                Pnames[i],".txt",sep=""))) {
                file.remove(paste(fileResults, "/EnvisageResults/", 
                    Pnames[i],".txt",sep=""))
            }
            write.table(results[[i]],file=paste(fileResults,"/EnvisageResults/", 
                Pnames[i],".txt",sep=""), append=F, 
                row.names=T, col.names=FALSE, quote=FALSE, sep="\t")
        }
        rm(i)
    } else if (length(results) == 0) {
      warning <- tkmessageBox(message=paste("Warning: No genes found to ",
          "be significant.", sep=""), icon="warning", type="ok") 
    }

    ## Output information for error genes
    list <- NULL
    for (i in 1:nrow(exprData)) {
        if (!is.na(errorGenes[[i]][1])) list <- c(list, i)
    }
    rm(i)

    if (length(list) > 0) {
        errorGenesShort <- errorGenes[list]

        ## Write gene list for errors
        if (file.exists(paste(fileResults, 
            "/EnvisageResults/AliasGenes.txt", sep=""))) {
            file.remove(paste(fileResults, 
                "/EnvisageResults/AliasGenes.txt", sep=""))
        }
        write.table(names(errorGenesShort),file=paste(fileResults,
            "/EnvisageResults/AliasGenes.txt", sep=""), append=F, 
            row.names=F, col.names=FALSE, quote=FALSE, sep="\t")
        ## Write error report
        if (file.exists(paste(fileResults, 
            "/EnvisageResults/AliasReport.txt", sep=""))) { 
            file.remove(paste(fileResults, 
                "/EnvisageResults/AliasReport.txt", sep=""))
        }
        for (i in 1:length(errorGenesShort)) {        
            write(names(errorGenesShort)[i], file=paste(fileResults,
                "/EnvisageResults/AliasReport.txt", sep=""), append=T)
            write("\nAliasing is seen across the following conditions:",
                file=paste(fileResults,
                "/EnvisageResults/AliasReport.txt", sep=""), append=T)
            for (j in 1:length(errorGenesShort[[i]])) {
                write(errorGenesShort[[i]][j], file=paste(fileResults,
                    "/EnvisageResults/AliasReport.txt", sep=""), append=T)
            }
            write("\n\n", file=paste(fileResults,
                "/EnvisageResults/AliasReport.txt", sep=""), append=T)
        } 
    }

    ## Write gene list for genes that fail the Anova
    if (length(failedAnova) > 0) {
        try(file.remove(paste(fileResults, 
            "/EnvisageResults/Failed.txt", sep="")))
        write.table(failedAnova,file=paste(fileResults,
            "/EnvisageResults/Failed.txt", sep=""), append=F, 
            row.names=F, col.names=FALSE, quote=FALSE, sep="\t")
    }
    return(results)    
}	

###############################################################################
#                        END - FUNCTION - result.output                       #
###############################################################################